@@ -2,10 +2,11 @@
 
 from typing import Iterator
 
+import numpy as np
 from mcipc.rcon.enumerations import Item
 
-from mcwb.types import Anchor, Direction, Number, Offsets, Profile, Row, Vec3
-
+from mcwb.types import (Anchor, Direction, Items, Number, Offsets,
+                        Profile, Row, Vec3)
 
 __all__ = ['get_direction', 'normalize', 'offsets', 'validate']
 
@@ -46,6 +47,14 @@ def get_direction(start: Vec3, end: Vec3) -> Vec3:
     return end - start
 
 
+def y_rotate(direction: Vec3, clockwise: bool = True) -> Vec3:
+    """rotate a cardinal direction about the y axis"""
+
+    current = Direction.cardinals.index(direction)
+    rotated = current + (1 if clockwise else -1)
+    return Direction.cardinals[rotated % len(Direction.cardinals)]
+
+
 def normalize(profile: Profile, default: Item = Item.AIR) -> Iterator[Row]:
     """Normalizes a profile."""
 
@@ -60,9 +69,6 @@ def offsets(profile: Profile, direction: Vec3, anchor: Anchor) -> Offsets:
     width = len(profile[0])
     x_start = y_start = 0
 
-    if isinstance(direction, Direction):
-        direction = direction.value
-
     if anchor in {Anchor.BOTTOM_LEFT, Anchor.BOTTOM_RIGHT}:
         y_start = height - 1
     if anchor in {Anchor.TOP_RIGHT, Anchor.BOTTOM_RIGHT}:
@@ -77,14 +83,47 @@ def offsets(profile: Profile, direction: Vec3, anchor: Anchor) -> Offsets:
             yield (block, vec3)
 
 
-def validate(profile: Profile) -> bool:
-    """Validates a profile to have rows with equal length."""
+def validate(items: Items):
+    """
+    Validates a row, profile or cuboid, ensuring consistent dimensions
+    """
+    ragged = False
 
-    rows = iter(profile)
+    # np ragged array is deprecated but only supplys a warning currently
+    with np.warnings.catch_warnings(record=True) as warning:
+        np.warnings.simplefilter("always")
+        narray = np.array(items)
+        if warning and 'ragged nested' in str(warning[-1].message):
+            ragged = True
 
-    try:
-        first = len(next(rows))
-    except StopIteration:
-        return True
+    return int(narray.ndim) if narray.dtype == Item and not ragged else 0
 
-    return all(len(row) == first for row in rows)
+
+def shift(arr: np.ndarray, vec: Vec3, fill: Item = Item.AIR) -> np.ndarray:
+    """ shift a 3d array of Item by vec, discarding the cells that are
+        shifted out and filling the new space with Item fill
+    """
+
+    # this is the fastest approach in python, see 1d Benchmark at
+    # https://stackoverflow.com/questions/30399534/shift-elements-in-a-numpy-array
+    result: np.ndarray = np.array(np.full_like(arr, fill), dtype=Item)
+    if vec.y > 0:
+        result[:, : vec.y, :] = fill
+        result[:, vec.y:, :] = arr[:, : -vec.y, :]
+    elif vec.y < 0:
+        result[:, vec.y:, :] = fill
+        result[:, : vec.y, :] = arr[:, -vec.y:, :]
+    if vec.x > 0:
+        result[: vec.x, :, :] = fill
+        result[vec.x:, :, :] = arr[: -vec.x, :, :]
+    elif vec.x < 0:
+        result[vec.x:, :, :] = fill
+        result[: vec.x, :, :] = arr[-vec.x:, :, :]
+    if vec.z > 0:
+        result[:, :, : vec.z:] = fill
+        result[:, :, vec.z:] = arr[:, :, : -vec.z]
+    elif vec.z < 0:
+        result[:, :, vec.z:] = fill
+        result[:, :, : vec.z] = arr[:, :, -vec.z:]
+
+    return result
